import java.util.*;
import java.util.logging.*;

/**
 * This class represents the alignment algorithm. The algorithm is implemented
 * in the <code>align</code> method.
 * 
 * @author darko
 * 
 */
public class Aligner {
	protected Context context;

	Logger logger = Logger.getLogger(Aligner.class.getName());

	public Aligner(Context context) {
		this.context = context;
	}

	public Context getContext() {
		return this.context;
	}

	public void setContext(Context context) {
		this.context = context;
	}

	public Alignment align(List<String> corpusNames, String hubCorpusName, List<String> alignmentNames) {
		Set<String> traversedSentences = new HashSet<String>();
		Set<String> traversedLinks = new HashSet<String>();

		String resultName = "";

		List<Corpus> corpora = new ArrayList<Corpus>();
		for (Iterator<String> iter = corpusNames.iterator(); iter.hasNext();) {
			String corpusName = iter.next();
			Corpus c = context.corpora.get(corpusName);
			if (c == null) {
				logger.warning("The corpus '" + corpusName + "' doesn't exist - will be ignored!");
				iter.remove();
				continue;
			}
			corpora.add(c);
			resultName += corpusName;
		}

		for (Iterator<String> iter = alignmentNames.iterator(); iter.hasNext();) {
			String alignmentName = iter.next();
			if (!context.alignments.containsKey(alignmentName)) {
				logger.warning("The alignment '" + alignmentName + "' doesn't exist - will be ignored!");
				iter.remove();
			}
		}

		Alignment alignment = new Alignment(resultName);
		alignment.corpora = corpora;
		alignment.n = corpora.size();

		Corpus hubCorpus = context.corpora.get(hubCorpusName);
		if (hubCorpus == null) {
			logger.severe("The corpus '" + hubCorpus + "' doesn't exist");
			throw new RuntimeException();

		}
		Queue<Sentence> sentenceQueue = new LinkedList<Sentence>();

		for (Map.Entry<String, Sentence> entry : hubCorpus.sentences.entrySet()) {
			String curID = entry.getKey();
			Sentence curSentence = entry.getValue();
			if (!traversedSentences.contains(curID)) {
				sentenceQueue.add(curSentence);
				Link link = new Link();
				for (String corpusName : corpusNames) {
					link.linkGroups.put(corpusName, new LinkedHashSet<Sentence>());
				}
				while (!sentenceQueue.isEmpty()) {
					Sentence workingSentence = sentenceQueue.poll();
					traversedSentences.add(workingSentence.id);
					for (String alignmentName : alignmentNames) {
						Link workingLink = workingSentence.links.get(alignmentName);
						if (workingLink != null && !traversedLinks.contains(workingLink.string)) {
							traversedLinks.add(workingLink.string);
							for (String corpusName : corpusNames) {
								Set<Sentence> linkedSentences = workingLink.linkGroups.get(corpusName);
								if (linkedSentences != null) {
									link.linkGroups.get(corpusName).addAll(linkedSentences);
								}
							}
							Set<Sentence> linkedHubs = workingLink.linkGroups.get(hubCorpusName);
							for (Sentence s : linkedHubs) {
								if (!traversedSentences.contains(s)) {
									sentenceQueue.add(s);
								}
							}
						}
					}
				}
				alignment.links.add(link);
			}
		}
		return alignment;
	}
}
