import java.util.logging.*;
import java.io.*;

import com.martiansoftware.jsap.*;

/**
 * A parser for the command line arguments.
 * @author darko
 *
 */
public class CLIParser {
	Logger logger = Logger.getLogger(CLIParser.class.getName());
	
	protected JSAP jsap = new JSAP();
	protected JSAPResult result = null;

	public CLIParser() {

		try {
			/* corpora */
			FlaggedOption corpora = new FlaggedOption("corpora")	.setLongFlag("corpora")
			                                                		.setShortFlag('c')
			                                                		.setList(true)
			                                                		.setListSeparator(',')
			                                                		.setStringParser(JSAP.STRING_PARSER)
			                                                		.setRequired(false)
			                                                		.setUsageName("corpus");
			corpora.setHelp("List of corpora to align (Default: align all)");
			jsap.registerParameter(corpora);
			
			/* hub */
			FlaggedOption hub = new FlaggedOption("hub")	.setLongFlag("hub")
																		.setShortFlag('h')
																		.setStringParser(JSAP.STRING_PARSER)
																		.setRequired(true);
			hub.setHelp("The name of the hub corpus");
			jsap.registerParameter(hub);
			
			
			/* alignments */
			FlaggedOption alignments = new FlaggedOption("aligns")	.setLongFlag("aligns")
			                                                   		.setShortFlag('a')
			                                                   		.setList(true)
			                                                   		.setListSeparator(',')
			                                                   		.setStringParser(JSAP.STRING_PARSER)
			                                                   		.setRequired(false)
			                                                   		.setUsageName("align");
			alignments.setHelp("List of alignments to use (Default: use all)");
			jsap.registerParameter(alignments);
			
			/* output */
			FlaggedOption output = new FlaggedOption("output")	.setLongFlag("output")
																				.setShortFlag('o')
																				.setStringParser(JSAP.STRING_PARSER)
																				.setRequired(false);
			output.setHelp("The name of the output file (Default: <alignment_name>.xml)");
			jsap.registerParameter(output);
			
			
			/* files */
			UnflaggedOption files = new UnflaggedOption("files")	.setStringParser(JSAP.STRING_PARSER)
                                                      			.setRequired(true)
                                                      			.setGreedy(true)
                                                      			.setUsageName("inputFile");
			files.setHelp("List of input alignment files");
			jsap.registerParameter(files);
			
		} catch (Exception ex) {
			logger.severe(ex.toString());
			ex.printStackTrace();
			throw new RuntimeException();
		}

	}
	
	public void parse(String[] args) {
		result = jsap.parse(args);
		if (!result.success()) {
         
         System.err.println();

         for (java.util.Iterator errs = result.getErrorMessageIterator();
                 errs.hasNext();) {
             System.err.println("Error: " + errs.next());
         }
         printHelp(System.err);
         System.exit(1);
     }
	}
	
	public String[] getFileNames() {
		return result.getStringArray("files");
	}
	public String[] getCorporaNames() {
		return result.getStringArray("corpora");
	}
	public String[] getAlignmentNames() {
		return result.getStringArray("aligns");
	}
	public String getOutputFilename() {
		return result.getString("output");
	}
	public String getHubCorpus() {
		return result.getString("hub");
	}
	
	private void printHelp(PrintStream out) {
		
      String jarName = CLIParser.class.getProtectionDomain().getCodeSource().getLocation().getPath();
      out.println();
		out.println("Usage: java -jar " + jarName.substring(jarName.lastIndexOf('/') + 1) + " OPTIONS " + jsap.getByID("files").getSyntax());
		out.println();
		out.println("OPTIONS:");
		out.println(jsap.getHelp(80, ""));
	
	}
}
