import java.util.*;

/**
 * The main class of the application. Represents the Command line interface of
 * the program. This is the starting point of the application when executing
 * from shell.
 * 
 * @author darko
 * 
 */
public class Main
		extends Object {

	public static void main(String[] args) {
		CLIParser cliParser = new CLIParser();
		cliParser.parse(args);

		String[] files = cliParser.getFileNames();
		String[] corpora = cliParser.getCorporaNames();
		String[] aligns = cliParser.getAlignmentNames();
		String output = cliParser.getOutputFilename();
		String hub = cliParser.getHubCorpus();

		AlignmentParser alignParser = new AlignmentParser("untitled");
		alignParser.parse(files);

		Context context = alignParser.getContext();

		List<String> corpusNames;
		if (corpora.length == 0) {
			corpusNames = new ArrayList<String>(context.corpora.keySet());
		} else {
			corpusNames = Arrays.asList(corpora);
		}

		List<String> alignmentNames;
		if (aligns.length == 0) {
			alignmentNames = new ArrayList<String>(context.alignments.keySet());
		} else {
			alignmentNames = Arrays.asList(aligns);
		}

		Aligner aligner = new Aligner(context);
		Alignment result = aligner.align(corpusNames, hub, alignmentNames);

		if (output == null) {
			output = result.name + ".xml";
		}
		AlignmentWriter writer = new AlignmentWriter(output, result);
		writer.write();

	}

}
